@@ -106,6 +106,7 @@ void gaa_help()
 	__gaa_helpsingle('v', "verbose", "", "some more information is displayed.");
 	__gaa_helpsingle('q', "quiet", "", "suppress some non critical warnings.");
 	__gaa_helpsingle(0, "nosalt", "", "Do not use salt.");
+	__gaa_helpsingle(0, "time", "", "Show time elapsed.");
 	__gaa_helpsingle('m', "hmac", "", "Use password to hash (hmac).");
 	__gaa_helpsingle('d', "doublecheck", "", "Double check passwords.");
 	__gaa_helpsingle('c', "check", "", "check files.");
@@ -135,32 +136,34 @@ typedef struct _gaainfo gaainfo;
 
 struct _gaainfo
 {
-#line 52 "shash.gaa"
+#line 55 "shash.gaa"
 	char **input;
-#line 51 "shash.gaa"
+#line 54 "shash.gaa"
 	int size;
-#line 42 "shash.gaa"
+#line 45 "shash.gaa"
 	char *keys;
-#line 37 "shash.gaa"
+#line 40 "shash.gaa"
 	int nolock;
-#line 33 "shash.gaa"
+#line 36 "shash.gaa"
 	char *algorithm;
-#line 30 "shash.gaa"
+#line 33 "shash.gaa"
 	char *config_file;
-#line 29 "shash.gaa"
+#line 32 "shash.gaa"
 	int config;
-#line 25 "shash.gaa"
+#line 28 "shash.gaa"
 	char *kmode;
-#line 22 "shash.gaa"
+#line 25 "shash.gaa"
 	int text;
-#line 19 "shash.gaa"
+#line 22 "shash.gaa"
 	int binary;
-#line 16 "shash.gaa"
+#line 19 "shash.gaa"
 	int check;
-#line 13 "shash.gaa"
+#line 16 "shash.gaa"
 	int double_check;
-#line 10 "shash.gaa"
+#line 13 "shash.gaa"
 	int hmac;
+#line 10 "shash.gaa"
+	int show_time;
 #line 7 "shash.gaa"
 	int nosalt;
 #line 3 "shash.gaa"
@@ -219,7 +222,7 @@ int gaa_error = 0;
 #define GAA_MULTIPLE_OPTION     3
 
 #define GAA_REST                0
-#define GAA_NB_OPTION           18
+#define GAA_NB_OPTION           19
 #define GAAOPTID_license	1
 #define GAAOPTID_version	2
 #define GAAOPTID_help	3
@@ -235,9 +238,10 @@ int gaa_error = 0;
 #define GAAOPTID_check	13
 #define GAAOPTID_doublecheck	14
 #define GAAOPTID_hmac	15
-#define GAAOPTID_nosalt	16
-#define GAAOPTID_quiet	17
-#define GAAOPTID_verbose	18
+#define GAAOPTID_time	16
+#define GAAOPTID_nosalt	17
+#define GAAOPTID_quiet	18
+#define GAAOPTID_verbose	19
 
 #line 168 "gaa.skel"
 
@@ -497,6 +501,7 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECK1STR("c", GAAOPTID_check);
 			GAA_CHECK1STR("d", GAAOPTID_doublecheck);
 			GAA_CHECK1STR("m", GAAOPTID_hmac);
+			GAA_CHECK1STR("", GAAOPTID_time);
 			GAA_CHECK1STR("", GAAOPTID_nosalt);
 			GAA_CHECK1STR("q", GAAOPTID_quiet);
 			GAA_CHECK1STR("v", GAAOPTID_verbose);
@@ -519,6 +524,7 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECKSTR("check", GAAOPTID_check);
 			GAA_CHECKSTR("doublecheck", GAAOPTID_doublecheck);
 			GAA_CHECKSTR("hmac", GAAOPTID_hmac);
+			GAA_CHECKSTR("time", GAAOPTID_time);
 			GAA_CHECKSTR("nosalt", GAAOPTID_nosalt);
 			GAA_CHECKSTR("quiet", GAAOPTID_quiet);
 			GAA_CHECKSTR("verbose", GAAOPTID_verbose);
@@ -560,28 +566,28 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
     {
 	case GAAOPTID_license:
 	OK = 0;
-#line 49 "shash.gaa"
+#line 52 "shash.gaa"
 { hash_license(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_version:
 	OK = 0;
-#line 48 "shash.gaa"
+#line 51 "shash.gaa"
 { hash_version(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_help:
 	OK = 0;
-#line 47 "shash.gaa"
+#line 50 "shash.gaa"
 { gaa_help(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_listkeygen:
 	OK = 0;
-#line 46 "shash.gaa"
+#line 49 "shash.gaa"
 { list_keygen_algorithms(); exit(0); ;};
 
 		return GAA_OK;
@@ -591,21 +597,21 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_key.arg1, gaa_getstr, GAATMP_key.size1);
 		gaa_index++;
-#line 43 "shash.gaa"
+#line 46 "shash.gaa"
 { gaaval->keys = GAATMP_key.arg1; ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_list:
 	OK = 0;
-#line 40 "shash.gaa"
+#line 43 "shash.gaa"
 { hash_list(); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_nolock:
 	OK = 0;
-#line 38 "shash.gaa"
+#line 41 "shash.gaa"
 { gaaval->nolock = 1 ;};
 
 		return GAA_OK;
@@ -615,7 +621,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_algorithm.arg1, gaa_getstr, GAATMP_algorithm.size1);
 		gaa_index++;
-#line 34 "shash.gaa"
+#line 37 "shash.gaa"
 { gaaval->algorithm = GAATMP_algorithm.arg1 ;};
 
 		return GAA_OK;
@@ -625,7 +631,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_config.arg1, gaa_getstr, GAATMP_config.size1);
 		gaa_index++;
-#line 31 "shash.gaa"
+#line 34 "shash.gaa"
 { gaaval->config=1; if (GAATMP_config.size1==1) gaaval->config_file = GAATMP_config.arg1 ;};
 
 		return GAA_OK;
@@ -635,44 +641,51 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_keymode.arg1, gaa_getstr, GAATMP_keymode.size1);
 		gaa_index++;
-#line 26 "shash.gaa"
+#line 29 "shash.gaa"
 { gaaval->kmode = GAATMP_keymode.arg1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_text:
 	OK = 0;
-#line 23 "shash.gaa"
+#line 26 "shash.gaa"
 { gaaval->text = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_binary:
 	OK = 0;
-#line 20 "shash.gaa"
+#line 23 "shash.gaa"
 { gaaval->binary = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_check:
 	OK = 0;
-#line 17 "shash.gaa"
+#line 20 "shash.gaa"
 { gaaval->check = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_doublecheck:
 	OK = 0;
-#line 14 "shash.gaa"
+#line 17 "shash.gaa"
 { gaaval->double_check = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_hmac:
 	OK = 0;
-#line 11 "shash.gaa"
+#line 14 "shash.gaa"
 { gaaval->hmac = 1 ;};
 
+		return GAA_OK;
+		break;
+	case GAAOPTID_time:
+	OK = 0;
+#line 11 "shash.gaa"
+{ gaaval->show_time = 1 ;};
+
 		return GAA_OK;
 		break;
 	case GAAOPTID_nosalt:
@@ -698,7 +711,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		break;
 	case GAA_REST:
 		GAA_OPTIONALLIST_FILL(GAAREST_tmp.arg1, gaa_getstr, char*, GAAREST_tmp.size1);
-#line 53 "shash.gaa"
+#line 56 "shash.gaa"
 { gaaval->input = GAAREST_tmp.arg1; gaaval->size = GAAREST_tmp.size1 ;};
 
 		return GAA_OK;
@@ -726,11 +739,11 @@ int gaa(int argc, char **argv, gaainfo *gaaval)
     if(inited == 0)
     {
 
-#line 55 "shash.gaa"
+#line 58 "shash.gaa"
 { gaaval->quiet=1; gaaval->config=0; gaaval->kmode = NULL; gaaval->hmac = 0;
        gaaval->algorithm=NULL; gaaval->keys=NULL; gaaval->config_file=NULL; gaaval->input=NULL;
        gaaval->double_check=0; gaaval->nolock=0; gaaval->nosalt=0; gaaval->check=0; gaaval->text=0; 
-       gaaval->binary=0; ;};
+       gaaval->binary=0; gaaval->show_time=0; ;};
 
     }
     inited = 1;
