@@ -29,6 +29,7 @@
 #include <locks.h>
 #include <random.h>
 #include <environ.h>
+#include <time.h>
 
 #ifdef HAVE_CTYPE_H
 #include <ctype.h>
@@ -47,6 +48,7 @@ int total = 0;
 int nosalt = FALSE;
 int double_check = FALSE;
 int hmac = 0;
+int show_time = 0;
 
 int main(int argc, char **argv)
 {
@@ -115,6 +117,7 @@ int main(int argc, char **argv)
 	nolock = info.nolock;
 	hmac = info.hmac;
 	double_check = info.double_check;
+	show_time = info.show_time;
 
 	if (info.text == TRUE) {
 		binary = FALSE;
@@ -307,12 +310,79 @@ void view(char *desc, void *x, int sizeofx)
 
 }
 
+
+#ifdef HAVE_FTIME
+# include <sys/timeb.h>
+#else
+# include <time.h>
+#endif
+
+/* Accepts time elapsed in milliseconds and file size in bytes.
+ */
+static void show_stats( time_t ms_time, size_t file_size) {
+time_t time = ms_time/1000;
+time_t millitm = ms_time % 1000;
+int show = 0;
+
+	fprintf(stderr, "Time needed: ");
+	
+	if (time/3600 > 0) {
+		fprintf(stderr, "%lu hours", time/3600);
+		show = 1;
+	}
+	
+	if ((time/60)-( (time/3600)*60 ) > 0) {
+		if (show>0) fprintf(stderr, ", ");
+		fprintf( stderr, "%lu minutes", (time/60)-(time/3600) );
+		show = 1;
+	}
+
+	if (time - ((time/60)+((time/3600)*60)*60 ) > 0) {
+		if (show>0) fprintf(stderr, ", ");
+		fprintf( stderr, "%lu seconds", time-((time/60)+(time/3600)) );
+		show = 1;
+	}
+	
+	if (millitm > 0) {
+		if (show>0) fprintf(stderr, ", ");
+		fprintf( stderr, "%lu milliseconds", millitm);
+		show = 1;
+	}
+	
+	if (show==0)
+		fprintf(stderr, "Wow, no time counted!\n");
+	else {
+		fprintf(stderr, ".\n");
+	
+		if (file_size > 1000000) {
+			fprintf(stderr, "Hashed %.3f MB, at a %.3f MB/sec rate (1 MB == 1000000 Bytes).\n", 
+				(float)( (double)file_size/(double)1000000),
+				(float)( (( (double) file_size/ (double)1000000)/ (double)ms_time)*(double)1000));
+		} else { /* KB */
+			fprintf(stderr, "Hashed %.3f KB, at a %.3f KB/sec rate (1 KB == 1000 Bytes).\n", 
+				(float)( (double)file_size/(double)1000),
+				(float)( (( (double) file_size/ (double)1000)/ (double)ms_time)*(double)1000));
+		}
+	}
+}
+
+
+
+#define BUFFER_SIZE 32*1024
 void *sumfile(char *filename, hashid algorithm, char *password, int plen)
 {
 	FILE *file;
-	char buffer[4096];
+	char buffer[BUFFER_SIZE];
 	MHASH td;
 	int x;
+#ifdef HAVE_FTIME
+	struct timeb start_time;
+	struct timeb end_time;
+#else
+	time_t start_time;
+	time_t end_time;
+#endif
+	size_t file_sum;
 
 
 	if (filename != NULL) {
@@ -354,29 +424,68 @@ void *sumfile(char *filename, hashid algorithm, char *password, int plen)
 
 	if (td == MHASH_FAILED) {
 		sprintf(msg, "Internal library error.\n");
+		fclose(file);
 		err_crit(msg);
 		return NULL;
 	}
 
+#ifdef HAVE_FTIME
+	ftime(&start_time);
+#else
+	start_time = time(0);
+#endif
+	
+	file_sum = 0;
+	
 	for (;;) {
-		x = fread(buffer, 1, 4096, file);
+		clearerr( file);
+		x = fread(buffer, 1, BUFFER_SIZE, file);
+		file_sum += x;
+
 		if (x == 0) {
-			if (ferror(file) == 0) {
-				break;
-			} else {
-				sprintf(msg, "%s: Error in file.\n",
-					filename);
-				err_crit(msg);
+			if (ferror(file)!=0) {
+				fprintf(stderr, "Error in file\n");
 				fclose(file);
 				return NULL;
 			}
+			break;
 		}
 		mhash(td, &buffer, x);
 	}
 
+#ifdef HAVE_FTIME
+	ftime(&end_time);
+#else	
+	end_time = time(0);
+#endif
+	
 	unlock(fileno(file));
 	fclose(file);
 
+	if (show_time!=0) {
+#ifdef HAVE_FTIME
+		end_time.time*=1000;
+		end_time.time+=end_time.millitm;
+
+		start_time.time*=1000;
+		start_time.time+=start_time.millitm;
+		
+		end_time.time -= start_time.time;
+		
+		start_time.time = end_time.time;
+		
+		end_time.millitm = end_time.time % 1000;
+
+		show_stats( end_time.time, file_sum);
+
+#else
+		end_time -= start_time;
+
+		show_stats( end_time, file_sum);
+
+#endif
+	}
+
 	if (password == NULL) {
 		return mhash_end(td);
 	} else {
@@ -627,20 +736,20 @@ int checkfile(char *filename, hashid algorithm, char *keyword, int plen,
 
 void hash_license()
 {
-	fprintf(stdout, ("\nCopyright (C) 1999 Nikos Mavroyanopoulos\n\
-This program is free software; you can redistribute it and/or modify \n\
-it under the terms of the GNU General Public License as published by \n\
-the Free Software Foundation; either version 2 of the License, or \n\
-(at your option) any later version. \n\
-\n\
-This program is distributed in the hope that it will be useful, \n\
-but WITHOUT ANY WARRANTY; without even the implied warranty of \n\
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n\
-GNU General Public License for more details. \n\
-\n\
-You should have received a copy of the GNU General Public License \n\
-along with this program; if not, write to the Free Software \n\
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
+	fprintf(stdout, ("\nCopyright (C) 1999 Nikos Mavroyanopoulos\n"
+"This program is free software; you can redistribute it and/or modify \n"
+"it under the terms of the GNU General Public License as published by \n"
+"the Free Software Foundation; either version 2 of the License, or \n"
+"(at your option) any later version. \n"
+"\n"
+"This program is distributed in the hope that it will be useful, \n"
+"but WITHOUT ANY WARRANTY; without even the implied warranty of \n"
+"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n"
+"GNU General Public License for more details. \n"
+"\n"
+"You should have received a copy of the GNU General Public License \n"
+"along with this program; if not, write to the Free Software \n"
+"Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 }
 
 void hash_version()
